/*==========================================================================
    CGL equation
      ∂t w =c0*w-c2*norm(w)*w+c1*△w

      v = spatial average of w(x,y);
      usually, c0=1,real(c1)=real(c2)=1.0;
===========================================================================*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <XRS/Complex_Control.h>
#include <XRS/Graphic.h>
using namespace XRS;

#define  Random   drand48()
#define  Dispstep 2
#define  Color    12


/************** 大域変数と関数宣言 ********************/

double    c0=0,c1=2,c2=-2; // パラメータ
complex<double> **w;                        // 場
double    L;                                // システムサイズ
double    t;                                // 時間

void            initial(int);
void            boundary();


/*---------------- 以下 XWindow 用 --------------------*/

Complex_Control_Window CW;
Graphic_Window  W,SW;
int             Wimax,Wjmax;
int             Initial=0;
double          Amp=0.8;

unsigned long*  colordisk();
int             diskcolor(complex<double> w);
void            draw();
void            SW_draw();
void            explain(int)
{  if(!fork()){ execlp("evince","evince","README_CGL.pdf",NULL);}  }



int main(int argc,char **argv)
{
   double    DT,DX;
   complex<double> **w2,dw,t0,t1,t2;

   /*--------------- initialize ---------------------*/

   int ar=0;
   ar++; if(argc>ar) L=atof(argv[ar]);  else L=200;
   ar++; if(argc>ar) DX=atof(argv[ar]); else DX=1;
   ar++; if(argc>ar) DT=atof(argv[ar]); else DT=0.02;
   ar++; if(argc>ar) c0=atof(argv[ar]); else c0=0;
   ar++; if(argc>ar) c1=atof(argv[ar]); else c1=2;
   ar++; if(argc>ar) c2=atof(argv[ar]); else c2=-2;

   fprintf(stderr,"usage%% CGL [L] [DX] [DT] [c0] [c1] [c2]\n"
          " current systemsize: L=%g\n"
          " current parameter : c0=%g c1=%g c2=%g\n"
	  " space and time meshsize: DX=%g DT=%g\n"
   	  ,L,c0,c1,c2,DX,DT); 

   Wimax=Wjmax=(int)(L/DX);
   array(w,Wimax,Wjmax);   
   array(w2,Wimax,Wjmax);   


   /*---------------- setup window -------------------*/
   
   CW.realtime=0;
       f_dialog d(CW,explain);
       d.resize(2*d.width(),d.height());
       d.string(20);
       d.title(" readme (Click here)");
   CW.realtime=1;
   
   CW<<t<<"t"<<initial<<"initial"<<PARAMETER
       << c0 << c1 << c2 << Amp
       <<"c0"<<"c1"<<"c2"<<"Amp";
   
   W.geometry(Wimax,Wjmax,1)<<"CGL snapshot"
       <<COLOR(colordisk(),(2*Color+1)*(2*Color+1));

   SW.geometry(2*Color+1,2*Color+1,4)<<"color sample"
       <<COLOR(colordisk(),(2*Color+1)*(2*Color+1));

   initialize();
   SW_draw();

   
   /*--------------- main roution -------------------*/

   initial(0);
   while(1){
      checkevent();

      t0=complex<double>(1.0,c0)*DT;
      t1=complex<double>(1.0,c1)*DT/(DX*DX);
      t2=complex<double>(1.0,c2)*DT;

      for(int ts=0; ts<Dispstep;ts++){

         for(int i=1; i<Wimax-1; i++)
	    for(int j=1; j<Wjmax-1; j++){
	       dw=w[i-1][j]+w[i+1][j]+w[i][j-1]+w[i][j+1]-4.0*w[i][j];
               w2[i][j]=w[i][j]+t1*dw;
	    }
	  
         for(int i=1; i<Wimax-1; i++)
	    for(int j=1; j<Wjmax-1; j++)
               w[i][j]=w2[i][j]
		      +(t0-t2*norm(w[i][j]))*w[i][j];

	 boundary();
      }
      
      t+=DT*Dispstep;
      draw();
   }
}


/**********************  初期条件 *************************/

void initial(int)
{  
   for(int i=0; i<Wimax; i++)
      for(int j=0; j<Wjmax; j++)
          w[i][j]=polar((double)Random,(double)(2*M_PI*Random));

   boundary();
   t=0; Initial=1;
}


/**********************  境界条件 *************************/

void boundary()
{
// periodic boundary condition
    
    for(int j=0;j<Wjmax;j++){
        w[0][j]=w[Wimax-2][j];
        w[Wimax-1][j]=w[1][j];
    }
    
    for(int i=0;i<Wimax;i++){
        w[i][0]=w[i][Wjmax-2];
        w[i][Wjmax-1]=w[i][1];
    }
}    


/********************** 書く *************************/

void draw(void)
{
    if(Initial){ W.clear(); Initial=0; }

    for(int i=0; i<Wimax; i++)
       for(int j=0; j<Wjmax; j++)
  	   W.point(diskcolor(Amp*w[i][j]),i,j);
}


void SW_draw()
{
    for(int i=0; i<2*Color+1; i++)
       for(int j=0; j<2*Color+1; j++)
  	   SW.point(i+j*(2*Color+1),i,2*Color-j);
}


/********************** 色の配列 ***********************/

unsigned long* colordisk()
{
    unsigned long *cp=new unsigned long [(2*Color+1)*(2*Color+1)];
    for(int i=-Color; i<Color+1; i++)
        for(int j=-Color; j<Color+1; j++){
            int k=i+Color+(j+Color)*(2*Color+1);
            double r=sqrt((double)(i*i+j*j))/(double)Color;
	    double phi;
            if(i==0 && j==0) phi=0; 
            else             phi=atan2((double)i,(double)j)/(2*M_PI);
            cp[k]=color_pixel(phi,r,1);
        }
    return cp;
}    


/*------------------- 配列の番号を返す ----------------*/

int diskcolor(complex<double> w)
{
    static int cn=2*Color+1;
    w+=complex<double>(1,1); w*=Color;
    return (int)real(w)+(int)imag(w)*cn;
}


