/*================================================================
    DLA(Diffusion Limitted Aggregation, 拡散律速凝集) 
=================================================================*/    

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <XRS/Control.h>
#include <XRS/Graphic.h>
using namespace XRS;

#define PI         M_PI
#define RAN        (drand48()<0.5 ? -1:1)   // -1,1 をランダムに出す


/************** 大域変数と関数宣言 ********************/

int      **site;
double   **jump;
int      n;                // 凝集した粒子数
int      x,y;              // 拡散粒子の位置
int      N;                // 格子のサイズの半分
int      rr0=0;            // 凝集体を作る粒子の最大半径
double   r1;               // 粒子の出発点の半径
int      rr2;              // 半径がsqrt(rr2)以上になったら拡散粒子を取り除く。
double   initial_jump=10;  // 周りに凝集体がない領域での1歩の幅


void       initial(int);
int        setR(int);
void       stick(int,int);
inline int RR(int x,int y){ int i=x-N,j=y-N; return i*i+j*j;} //中心からの距離
inline double R(int x,int y,int x0,int y0){    // (x,y),(x0,y0) の距離
            int i=x-x0,j=y-y0;  return sqrt(double(i*i+j*j)); }


/*---------------- 以下 XWindow 用 --------------------*/
Control_Window CW;
Graphic_Window Lattice;

void            explain(int)
{  if(!fork()){ execlp("evince","evince","README_DLA.pdf",NULL);}  }


int main(int argc,char **argv)
{
   int      rr;           // 拡散粒子の中心からの距離の二乗
   int      nn;           // =(N-1)*(N-1)
   double   r;

   /*--------------- initialize ---------------------*/
   
   if(argc>1) N=atoi(argv[1]); else N=200;
   fprintf(stderr,"usage%% DLA [N]\n"
	  " current systemsize=2*N: N=%d\n",N); 
   
   nn=(N-1)*(N-1);
   array(site,2*N,2*N);
   array(jump,2*N,2*N);

/*----- initialize ----------*/
   
   CW.realtime=0;
       f_dialog d(CW,explain);
       d.resize(2*d.width(),d.height());
       d.string(20);
       d.title(" readme (Click here)");
   CW.realtime=1;

   CW<<n<<"n"<<initial<<"initial";
   Lattice.geometry(2*N,2*N,1)<<"DLA";
   initialize();

/*----- main routine --------*/
   
   initial(0);

   double theta;
   while(1){
      checkevent();
      // 粒子を１つ作る。
      theta=2*PI*drand48();
      x=int(r1*cos(theta)+(double)N+0.5);
      y=int(r1*sin(theta)+(double)N+0.5);
      n++;
      
      while(1){
         theta=2*M_PI*drand48();
         r=jump[x][y]; 
	 if(r>2){   // 歩幅 r のランダムウォーク
   	   x=int((double)x+0.5+r*cos(theta));
	   y=int((double)y+0.5+r*sin(theta));
	 }else{     // 1格子分のランダムウォーク
           x+=RAN; y+=RAN;	   
	 }
	                 
         rr=RR(x,y);
         if(rr>rr2){ n--; break;}  // 遠くになり過ぎたら、取り除く。
         if(rr<nn){ 
            if(site[x+1][y]+site[x-1][y]+site[x][y+1]+site[x][y-1]){
               stick(x,y);    // 隣接格子点に粒子があったら、くっつく。
               break;
            }
         }
      }
      if(nn<rr2) initial(0);
   }
}   


/***************** 初期条件 **************************/

void initial(int nn){
   Lattice.clear();
   for(int i=0; i<2*N; i++)
     for(int j=0; j<2*N; j++){ site[i][j]=0;  jump[i][j]=initial_jump;}
   rr0=-1;  n=0;
   setR(0);
   
   switch(nn){
   case 2:
     for(int i=0; i<2*N; i++)
       for(int j=0; j<2*N; j++){
	 if(RR(i,j)<(N*N)/16){ stick(i,j); n++;}
       } break;
   case 3:
     stick(N-N/8,N); n++;
     stick(N+N/8,N); n++;
     break;
   default:				  
     stick(N,N); n++;
   }
}


/************* rr0,r1,r2,rr2 の変更 ********************/

int        setR(int rr){ 
   if(rr>=rr0){  
      rr0=rr;
      double r0=sqrt((double)rr);
      r1=(double)(1+r0);
      double r2=(int)(initial_jump+r1); rr2=r2*r2;
      return 1;  
   }
   return 0;
}


/************* 凝集のプロセス ********************/

#define min(x,y)  ((x)<(y)?(x):(y))
#define max(x,y)  ((x)>(y)?(x):(y))

void  stick(int x,int y){
   site[x][y]=1;
   setR(RR(x,y));   
   Lattice.draw_point(1,x,y);
   for(int i=max(0,x-(int)initial_jump); i<min(2*N,x+(int)initial_jump); i++)
       for(int j=max(0,y-(int)initial_jump); j<min(2*N,y+(int)initial_jump); j++){
	   double rr=R(i,j,x,y);
	   if(rr<jump[i][j]) jump[i][j]=rr;
       }
}  
