/*----------------------------------------------------------------------------

     Krisher & Mikhailov excitable system 
  
     τε∂t U = εε ∂xx U + f(u) - v 
         ∂t V =    D ∂xx V + u  - γv

	 f(u) = -u + H(u-b)
	 b = a + σ [ ∫dx(U+V) - S ]

-----------------------------------------------------------------------------*/


/*********** include **************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <XRS/Control.h>
#include <XRS/Plane.h>
using namespace XRS;

/*********** define ***************/

#define integer(x)  ((int)(floor(x)+floor(2*((x)-floor(x)))))


/*********** global variable ************/

double                  t;
double                  *u,*v;

double                  LX;
double                  DT;
double                  DX;
double                  quit_time=10000.0;

double                  tau=1.0,ep=0.125,ga=1.0/3.0;
double                  a=0.275,sigma=10,S=4,delta=0.01;
double                  D=1.0;
double                  r_start=7.0;
double                  u_start=2.0;

int                     i_max,j_max;

Display                 *d;
GC                      black,blue,white;


/********** function & class prototype *****************/

void                    initial(int);
void                    win_draw();
inline double f(double u,double x){ 
   return 0.5*(tanh((u-x)/delta)+tanh(x/delta))-u;
}

Control_Window          Cwin;
Plane_Window            win;


int main(int argc,char **argv)
{
     int ar=0;
     ar++; if(argc>ar) LX=atof(argv[ar]); else LX=100;
     ar++; if(argc>ar) DX=atof(argv[ar]); else DX=0.1;
     ar++; if(argc>ar) DT=atof(argv[ar]); else DT=0.0001;
     fprintf(stderr,"Usage: AM [LX] [DX] [DT]\n");
     fprintf(stderr," LX=%g, DX=%g, DT=%g\n",LX,DX,DT);     

     i_max = (int)(LX/DX);
     j_max = 100;

     u = new double [i_max+1];  double *u2 = new double [i_max+1];
     v = new double [i_max+1];  double *v2 = new double [i_max+1];
     
/*----- initialize ----------*/

     Cwin<< t << initial 
         <<"t"<<"initial"<<PARAMETER
	 << tau <<    ep   << D <<   ga  << a << sigma << S 
         <<"tau"<<"epsilon"<<"D"<<"gamma"<<"a"<<"sigma"<<"S";
     
     win.geometry(i_max,j_max)<<"u(red) and v(green)"<<COLOR();

     initialize();

     d=win.disp();
     black=win.bgc;
     white=win.wgc;
     blue =graphic_context(color_pixel("blue"));

     initial(0);

/*----- main routine --------*/

     double DVxx=D*DT/(DX*DX);
     double DUxx=ep*DT/(DX*DX*tau);
     double DU  =DT/(ep*tau);
     
     while(1){

         checkevent();
	 double sum=0; for(int i=1;i<i_max;i++) sum+=u[i]+v[i];
	 double b=a+sigma*(sum*DX-S);

         for(int i=1;i<i_max;i++){
             double du=u[i-1]+u[i+1]-2.0*u[i];
             double dv=v[i-1]+v[i+1]-2.0*v[i];

             u2[i]= u[i] + DUxx*du + DU*( f(u[i],b)-v[i] );
             v2[i]= v[i] + DVxx*dv + DT*( u[i] - ga*v[i] );
         }

         for(int i=1;i<i_max;i++){ v[i]=v2[i]; u[i]=u2[i]; }

         v[0]=v[i_max-1];
         v[i_max]=v[1];
         u[0]=u[i_max-1];
         u[i_max]=u[1];

         t = t + DT;
         if(t >= quit_time) break;

         win_draw();
    }
}


/*************** initialize function ************/

void initial(int)
{
    t=0;
    for(int i=0; i<i_max+1; i++){
       u[i] = 0.0;
       v[i] = 0.0;
    }

    for(int i=0; i<i_max; i++)
       if(fabs((double)i-(double)i_max/2)<(r_start/DX)) u[i] = u_start;
}


//************** window draw **************

void win_draw(void)
{
    static int *ju=new int [i_max];
    static int *jv=new int [i_max];

    for(int i=0; i<i_max; i++){
       int j=(int)((double)j_max*(0.75-0.7*u[i]/u_start));
       if(j!=ju[i]){
         win.erase_point(0,i,ju[i]);
         win.draw_point(0,i,j); ju[i]=j;
       }

       j=(int)((double)j_max*(0.75-0.7*v[i]/u_start));
       if(j!=jv[i]){
         win.erase_point(1,i,jv[i]); 
         win.draw_point(1,i,j); jv[i]=j;
       }
    }
}



