#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <math.h>
#include <XRS/Simple.h>
#include <XRS/Control.h>
using namespace XRS;

#define PI M_PI        // ߼Ψ
#define MAX(x,y)  ((x>y)?(x):(y))
#define MIN(x,y)  ((x<y)?(x):(y))


/****************************************************
                ѿؿ
****************************************************/

double   dt;                // ֹ

double   k=100.0;           // γҤΥХͷ
double   b=2.0,R=1.0;       // γҤȤζߺ
double   c=0.0;             // ξüγƱΤζߺ
double   Q,T=1.0;           // Ǯѥ᡼겹 
double   xi;                // MD񹳷
double   K;                 // (γαưͥ륮)

double   t;                 // 
int      N;                 // γҿ 
double   *x,*y,*vx,*vy,*fx,*fy;    // γҤΰ֡®١
double   *vx_,*vy_,*fx_,*fy_;

void     init(int);         // γҤν
void     interaction(void); // γҤƯϤη׻
void     move(void);        // γҤαư
void     plot(int);         // 

void     debug(char mesg[126])
{
  printf("t=%g [%s]\n",t,mesg);
  for(int n=0; n<N; n++){ 
    printf("%d: (%g %g) (%g %g) (%g %g)\n",
	   n,x[n],y[n],vx[n],vy[n],fx[n],fy[n]);
  }
}

/*******************************************************
                   ᥤ 
*******************************************************/

int main(int argc,char **argv)
{
  if(argc>1) N=atoi(argv[1]);  else N=30;
  if(argc>2) dt=atof(argv[2]); else dt=0.001;
  if(argc>3) Q=atof(argv[3]);  else Q=0.001;  

  fprintf(stderr,"N=%d dt=%g Q=%g\n",N,dt,Q);
  fprintf(stderr,"K=%g b=%g R=%g T=%g\n",K,b,R,T);

  x=new double [N];   y=new double [N];
  vx=new double [N];  vy=new double [N];
  fx=new double [N];  fy=new double [N];
  vx_=new double [N]; vy_=new double [N];
  fx_=new double [N]; fy_=new double [N];

  init(0);                   // γҤν

  while(1){
    plot(0);                 // 
    interaction();           // γҤƯϤη׻
    move();                  // 1ƥåפλȯŸ

    K=0; for(int n=1; n<N; n++) K+=vx[n]*vx[n]+vy[n]*vy[n]; 
    K/=2*(N-1);
    
    xi+=dt*(K-T)*2.0*(N-1.0)/Q;  // MD 񹳷Ĵ
    t+=dt;
  } 
}

void init(int)
{
  for(int n=0; n<N; n++){ 
    x[n]=(double)n;  y[n]=0.1*(drand48()-0.5); 
    vx[n]=vy[n]=0; 
    vx_[n]=vy_[n]=0; 
    fx[n]=fy[n]=fx_[n]=fy_[n]=0;
  }
  vx[0]=vy[0]=vx_[0]=vy_[0]=0.0; 
  xi=0;  t=0;
  plot(1);
}


void interaction(void)
{
  for(int n=0; n<N; n++){ 
    fx[n]=-xi*vx[n];
    fy[n]=-xi*vy[n];
  }

  double x1,y1,r1,f1,fx1,fy1;
  for(int n=0; n<N-1; n++){ 
    x1=x[n+1]-x[n]; y1=y[n+1]-y[n];  
    r1=sqrt(x1*x1+y1*y1); x1/=r1; y1/=r1;
    f1=k*(r1-1.0);
    fx1   =f1*x1;  fy1   =f1*y1;
    fx[n]  +=fx1;  fy[n]  +=fy1;
    fx[n+1]-=fx1;  fy[n+1]-=fy1; 
  }

  double x2,y2,r2,fx2,fy2,f2;
  for(int n=0; n<N-2; n++){ 
    x2=x[n+2]-x[n]; y2=y[n+2]-y[n];  
    r2=sqrt(x2*x2+y2*y2);  x2/=r2; y2/=r2; r2/=R;
    f2=-b/(r2*r2+1.0);
    fx2   =f2*x2;   fy2   =f2*y2;
    fx[n]  +=fx2;   fy[n]  +=fy2;
    fx[n+2]-=fx2;   fy[n+2]-=fy2; 
  }

  double xN,yN,rN,fxN,fyN,fN;
  xN=x[N-1]-x[0]; yN=y[N-1]-y[0];  
  rN=sqrt(xN*xN+yN*yN);  xN/=rN; yN/=rN; rN/=R;
  fN=-c/(rN*rN+1.0);
  fxN   =fN*xN;   fyN   =fN*yN;
  fx[0]  +=fxN;   fy[0]  +=fyN;
  fx[N-1]-=fxN;   fy[N-1]-=fyN; 
}


void move(void)
{
  double dt2=dt*0.5, dt3=dt*1.5,dx,dy;

  for(int n=1; n<N; n++){ 
    vx[n]+=fx[n]*dt3-fx_[n]*dt2;   vy[n]+=fy[n]*dt3-fy_[n]*dt2;  
    dx=(vx[n]+vx_[n])*dt2;         dy=(vy[n]+vy_[n])*dt2; 
    vx_[n]=vx[n];                  vy_[n]=vy[n];   
    fx_[n]=fx[n];                  fy_[n]=fy[n];   
    x[n]+=dx;                      y[n]+=dy;
  }
}



/*******************************************************
             եå
********************************************************/

Simple_Window Win,TWin;
Control_Window CW,CWT(TWin);
Display  *d=Win.disp();
GC       gc0=graphic_context("blue");
GC       gc1=graphic_context("green");
GC       gc2=graphic_context("red");
GC       white=graphic_context("white");
GC       black=graphic_context("black");
double   Tamp=0.7;
int      width=500,Twidth=400,Theight=200,dispstep=10;

#define  TJ(y)  int(Theight*(1.0-Tamp*(y)))
#define  XMark(gc,x,y)  XFillArc(d,Win,gc,(x)-3,(y)-3,6,6,0,23040);


void plot(int reset)
{
  static int initial=-1, count=0, Ti=0, Tj0=0;
  static int *i=NULL,*j=NULL;
  int        Tj;

  if(initial<0){
    CW.title("parameters");
    CW<<t<<"t"<<K<<"K"<<init<<"initial"
      <<PARAMETER<<k<<"k"<<b<<"b"<<c<<"c"<<R<<"R"<<Q<<"Q"<<dt<<"dt";

    CWT<<PARAMETER<<T<<"T";

    Win.geometry(width,width)<<"snapshot";
    TWin.geometry(Twidth,Theight)<<"temerature";
    initialize();     
    initial=0;
  }

  if(reset){
    Win.clear();
    if(i!=NULL){ delete [] i; } i=new int [N]; 
    if(j!=NULL){ delete [] j; } j=new int [N]; 
  }
  
  checkevent(); 
  if(T<0){ T=0; CWT.draw();}

  if(count%dispstep==0){

    for(int n=0; n<N; n++)   XMark(white,i[n],j[n]);
    for(int n=0; n<N-1; n++) XDrawLine(d,Win,white,i[n],j[n],i[n+1],j[n+1]);

    for(int n=0; n<N; n++){
      i[n]=int(width*0.5*(1.0+x[n]/N));
      j[n]=int(width*0.5*(1.0-y[n]/N));
    }

    for(int n=0; n<N-1; n++) XDrawLine(d,Win,gc1,i[n],j[n],i[n+1],j[n+1]);
    for(int n=1; n<N-1; n++) XMark(gc0,i[n],j[n]);
    XMark(gc2,i[0],j[0]);  XMark(gc2,i[N-1],j[N-1]);

    
    Tj=TJ(K); 
    if(Tj>Twidth){ Tj=Twidth;}  if(Tj<0){ Tj=0;}
    XDrawLine(d,TWin,gc1,Ti,Tj0,Ti+1,Tj);
    Tj0=Tj;     
    Ti++; if(Ti>Twidth){TWin.clear(); Ti=0;}

    count=0;
  }

  count++;
}
