/*==================================================================================

     Maskse's model

     intial condition:  g[i]=0 or 1 randomly for i=0,1,2,...,L;

     (1) choose gr=0 or 1 randomly; i=L and h[i]+=size[gr]; 
     (2) For decreasing i, 
         if(h[i]-h[i-1]>zc[gr][g[i]]){ h[i-1]+=size[gr]; h[i]-=size[gr]; }
         else                        { g[i]=gr; }

     gr  : type of a rolling grain, 
     h[i]: height of a sandpile at the ith site,
     g[i]: type of the top grain at the ith site.

     size[i]=1.0-a*gr: height of a type i grain,
     zc[i][j]        : critical slope of a type i grain placeed on a type j grain.

         Stratification appears for zc[0][1]<zc[0][0]<zc[1][1]<zc[1][0]
         Segregation    appears for zc[0][1]<zc[1][1]<zc[0][0]<zc[1][0]

=================================================================================*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <boost/random.hpp>
using namespace std;
using namespace boost;
// 륻̥ĥˡ seed=4259 ǻȤȤ
// (URL) http://ja.wikipedia.org/wiki/륻̡ĥ
mt19937        gen(4259);
//0.01.0μ¿ΰפ
uniform_real<> dst(0.0,1.0);
variate_generator<mt19937, uniform_real<> > randreal( gen, dst );

#include <XRS/Array.h>
using namespace XRS;

int    L=50;
int    gr,s;
int    *g;
int    *h;

const int Type=2;
int    Size[Type]={1,2};
int    zc[Type][Type]={{4,8},{2,6}};
double ratio=0.8;
int    randtype(){ if(randreal()<ratio) return 0; else return 1; }

void   initial(int);
void   draw(int,int);


int main()
{ 
  g=new int [L+1];
  h=new int [L+1];

  initial(0);

  while(1){
    gr=randtype();
    s=Size[gr]; 
    h[L]+=s; draw(L,1);
    for(int i=L; i>0; i--){
      if(h[i]-h[i-1]>zc[gr][g[i]]){ h[i]-=s; h[i-1]+=s; draw(i-1,1); }
      else                        { g[i]=gr; draw(i,0); break; }
    }
    h[0]=0;
  }
}


void initial(int)
{
  draw(-1,0);
  for(int i=0; i<L+1; i++){ h[i]=0; g[i]=0;}
}


#include <XRS/Control.h>
#include <XRS/Simple.h>
Control_Window CW;
Simple_Window  W;
int    AmpX=4,AmpY=4;
int    width,height=700;
Display *d=W.disp();
GC     gc[Type],rc[Type],white;
int    I,J,dI,dJ;
struct timespec treq;
int    Wait_nsec=100000000;

void draw(int i,int rolling)
{
  static int Initial=1;

  if(Initial){
    treq.tv_sec = (time_t)0;
    width=(int)(AmpX*double(L+1));
    CW<<PARAMETER
      <<"ratio S/L"<<ratio
      <<"angle S/S"<<zc[0][0]<<"angle S/L"<<zc[0][1]
      <<"angle L/S"<<zc[1][0]<<"angle L/L"<<zc[1][1]
      <<"slow"<<Wait_nsec;
    W.geometry(width,height)<<"initial"<<initial;
    gc[0]=graphic_context("lawn green");
    gc[1]=graphic_context("dark orange");
    white=graphic_context("white");
    rc[0]=graphic_context("green");
    rc[1]=graphic_context("red");
    initialize();  
    Initial=0;
  }

  checkevent();

  if(i<0 || i>L) W.clear();  

  if(i<L && rolling){ XFillRectangle(d,W,white,I,J,dI,dJ); }

  I=AmpX*i;
  J=height-AmpY*h[i];
  dI=AmpX;
  dJ=AmpY*s;
  if(rolling) XFillRectangle(d,W,rc[gr],I,J,dI,dJ);
  else        XFillRectangle(d,W,gc[gr],I,J,dI,dJ);

  XFlush(d);
  treq.tv_nsec=Wait_nsec;
  nanosleep(&treq,NULL);
}
