/****************************************************************************
   project a point (X,Y,Z)
     to a point (x,y) on a surface whose z vector is (theta,phi).  
****************************************************************************/

#include <math.h>
	
struct axis{
   double  x[2][3];  /* cordinate of axis X,Y,Z ( i=0,1,2 ) */
   double  y[2][3];  /* (x[0][i],y[0][i])-(x[1][i],y[1][i]) */
};

struct projected_point{
   double  X,Y,Z;    
};


axis project(double theta,double phi,int N,
			 double X[],double Y[],double Z[],double x[],double y[])
{
double pai;
double xx,xy;
double zx,zy,zz;
double yx,yy,yz;
double Xmin,Xmax,Ymin,Ymax,Zmin,Zmax;
axis   a;
register int i;

pai=3.14159265358979;
phi=phi*pai;
theta=theta*pai;

xx=cos(phi); 
xy=sin(phi); 

zx=-sin(theta)*sin(phi);
zy=sin(theta)*cos(phi);
zz=cos(theta);

yx=-xy*zz;
yy=xx*zz;
yz=xy*zx-xx*zy;

Xmin=Xmax=Ymin=Ymax=Zmin=Zmax=0;
for(i=0; i<N; i++){
   x[i]=X[i]*xx+Y[i]*xy;                /* project (X[i],Y[i],Z[i]) */
   y[i]=X[i]*yx+Y[i]*yy+Z[i]*yz;        /*           to (x[i],y[i]) */
   if(X[i]>Xmax) Xmax=X[i];
   if(X[i]<Xmin) Xmin=X[i];
   if(Y[i]>Ymax) Ymax=Y[i];
   if(Y[i]<Ymin) Ymin=Y[i];
   if(Z[i]>Zmax) Zmax=Z[i];
   if(Z[i]<Zmin) Zmin=Z[i];
}

a.x[0][0]=Xmin*xx;   a.y[0][0]=Xmin*yx;  /* project (Xmin,0,0) */
a.x[1][0]=Xmax*xx;   a.y[1][0]=Xmax*yx;  /*         (Xmax,0,0) */
a.x[0][1]=Ymin*xy;   a.y[0][1]=Ymin*yy;  /*         (0,Ymin,0) */
a.x[1][1]=Ymax*xy;   a.y[1][1]=Ymax*yy;  /*         (0,Ymax,0) */
a.x[0][2]=0;         a.y[0][2]=Zmin*yz;  /*         (0,0,Zmin) */
a.x[1][2]=0;         a.y[1][2]=Zmax*yz;  /*         (0,0,Zmax) */

return(a);
}



projected_point project_xytoXY
(double theta,double phi, double Z, double x,double y)
{
double pai;
double xx,xy;
double zx,zy,zz;
double yx,yy,yz;
double det;
projected_point P;

pai=3.14159265358979;
phi=phi*pai;
theta=theta*pai;

xx=cos(phi); 
xy=sin(phi); 

zx=-sin(theta)*sin(phi);
zy=sin(theta)*cos(phi);
zz=cos(theta);

yx=-xy*zz;
yy=xx*zz;
yz=xy*zx-xx*zy;
det=xx*yy-xy*yx;

if(det==0)
{ fprintf(stderr,"ERROR in project.h: Determinant is zero!"); exit(0);}

P.X=( yy*x-xy*(y-Z*yz))/det;
P.Y=(-yx*x+xx*(y-Z*yz))/det;
P.Z=Z;

return (P);
}

  
