/*================================================================
    Lattice Gas model 
=================================================================*/    

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <XRS/Control.h>
#include <XRS/Graphic.h>
#include <XRS/Simple.h>
using namespace XRS;

#define PI         M_PI
#define RAND(p)    ((drand48()<(p))?1:0)

/************** 大域変数と関数宣言 ********************/

int   L,t;
int   seed;
double p;
int   **n,**n0,**n1;
int   **neigh;

void init(int);
void boundary();
void forces(int,int);
void draw(void); 


void            explain(int)
{  if(!fork()){ execlp("evince","evince","README_DLA.pdf",NULL);}  }


int main(int argc,char **argv)
{
   /*--------------- initialize ---------------------*/

   int ar=0;
   ar++; if(argc>ar) L   =atoi(argv[ar]); else L   =100;
   ar++; if(argc>ar) p   =atof(argv[ar]); else p   =0.1;
   ar++; if(argc>ar) seed=atoi(argv[ar]); else seed= 15;

   fprintf(stderr,"usage%% lattice_gas [L] [p] [seed]\n"
       " Current systemsize     =L   : L   =%d\n"
       " Threshhold of p        =p   : p   =%g\n"
       " Seed of random sequence=seed: seed=%d\n"
       ,L,p,seed); 
   
   array(n ,L+4,L+4);
   array(n0,L+4,L+4);
   array(n1,L+4,L+4);
   array(neigh,2,8);

   neigh[0][0]=1;     neigh[1][0]=0;
   neigh[0][1]=1;     neigh[1][1]=1;
   neigh[0][2]=0;     neigh[1][2]=1;
   neigh[0][3]=-1;    neigh[1][3]=1;
   neigh[0][4]=-1;    neigh[1][4]=0;
   neigh[0][5]=-1;    neigh[1][5]=-1;
   neigh[0][6]=0;     neigh[1][6]=-1;
   neigh[0][7]=1;     neigh[1][7]=-1;
  

/*----- main routine --------*/
   
   init(0);

   while(1){

      draw();

      for(int i=2; i<L+2; i++)
        for(int j=2; j<L+2; j++){ n1[i][j]=n0[i][j]=n[i][j]; }

      for(int i=2; i<L+2; i++)
        for(int j=2; j<L+2; j++) if(!n[i][j]) forces(i,j);

      for(int i=2; i<L+2; i++)
        for(int j=2; j<L+2; j++) n[i][j]=n0[i][j];

      boundary();

      t++;
   }
}   


/***************** 初期条件 **************************/

void init(int)
{
  srand48(seed);

  for(int i=0; i<L+4; i++)
    for(int j=0; j<L+4; j++) n1[i][j]=0;

  for(int i=2; i<L+2; i++)
    for(int j=2; j<L+2; j++) n1[i][j]=n[i][j]=RAND(p);

  boundary();
  t=0;
}


/***************** 境界条件 **************************/

void boundary(void)
{
  // 周期境界
  for(int i=2; i<L+2; i++){
      n[i][0]=n[i][L];    
      n[i][1]=n[i][L+1];     
      n[i][L+2]=n[i][2];  
      n[i][L+3]=n[i][3]; 
  }

  for(int j=2; j<L+2; j++){
      n[0][j]=n[L][j];    
      n[1][j]=n[L+1][j];     
      n[L+2][j]=n[2][j];  
      n[L+3][j]=n[3][j]; 
  }

  for(int i=0; i<L+4; i++){
      n[i][0]=n[i][L];    
      n[i][1]=n[i][L+1];     
      n[i][L+2]=n[i][2];  
      n[i][L+3]=n[i][3]; 
  }

  for(int j=0; j<L+4; j++){
      n[0][j]=n[L][j];    
      n[1][j]=n[L+1][j];     
      n[L+2][j]=n[2][j];  
      n[L+3][j]=n[3][j]; 
  }

}


/***************** 力の計算 **************************/

int Nint(double F, double ff)
{
  if(ff==0)       return 0;

  double t=F/ff;
  if     (t>0.5)  return -1;  
  else if(t<-0.5) return  1;  
  else            return  0;
}
 

void forces(int i0,int j0)
{
  double absforce[8];
  int    nlabel[8];

  int    nc=0,in,jn,nx,ny;
  double Fx,Fy,ff;

  for(int k=0; k<8; k++){
    in=i0+neigh[0][k];
    jn=j0+neigh[1][k];

    if(n[in][jn]){ 
      Fx=n[in+1][jn]-n[in-1][jn];
      Fy=n[in][jn+1]-n[in][jn-1];
      ff=sqrt(Fx*Fx+Fy*Fy);

      if(ff>0){
        nx=Nint(Fx,ff);
        ny=Nint(Fy,ff);

        if(nx==-neigh[0][k] && ny==-neigh[1][k]){
          nlabel[nc]=k;
          absforce[nc]=ff;
          nc++;
        }
      }
    }   
  }


  double fmax=-10.0; 
  int    mmax,ncmax;

  if(nc==0){
    return;

  }else if(nc==1){
    mmax=0;

  }else{
    for(int m=0; m<nc; m++)
       if(absforce[m] >= fmax){ fmax=absforce[m];  mmax=m;}
    
    ncmax=0;
    for(int m=0; m<nc; m++){ if(absforce[m] == fmax) ncmax++; }
    if(ncmax>1) return;

  }

  int k,ii,jj;
  k =nlabel[mmax];
  ii=i0+neigh[0][k];
  jj=j0+neigh[1][k];

  if(ii>L+1) ii-=L; else if(ii<2) ii+=L;
  if(jj>L+1) jj-=L; else if(jj<2) jj+=L;  

  n0[ii][jj]=0; n1[ii][jj]=0;
  n0[i0][j0]=1; n1[ii][jj]=2;

}


/***************** 描画 **************************/

Control_Window CW;
Graphic_Window Lattice;
GC             black=graphic_context("black");
int            mag=2,bw,b0=2*mag; 
long unsigned int  color[3]
  ={color_pixel("white"),color_pixel("green"),color_pixel("black")};


void draw()
{
  static int initial=1;
  
  if(initial){
    /*----- initialize ----------*/ 
    CW.realtime=0;
       f_dialog d(CW,explain);
       d.resize(2*d.width(),d.height());
       d.string(20);
       d.title(" readme (Click here)");
    CW.realtime=1;

    CW<<init<<"init"<<t<<"t"<<PARAMETER<<p<<"p";
    Lattice.geometry(L+4,L+4,mag)<<"Lattice Gas"<<COLOR(color,3);
    bw=L*mag;
    initialize();
    initial=0;
  } 

  checkevent();

  if(t%5==0){
    Lattice.draw_2D(n1);
    XDrawRectangle(Lattice.disp(),Lattice,black,b0,b0,bw,bw);
  }
}




